package com.your_domain.your_product.employee.dao;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;

import com.jdbaccess.da.DAFactory;
import com.jdbaccess.da.Select;
import com.jdbaccess.da.Result;
import com.jdbaccess.da.Transaction;
import com.jdbaccess.da.Update;
import com.jdbaccess.dao.DataAccessObjectImpl;
import com.jdbaccess.exception.ApplicationException;
import com.jdbaccess.sql.ComparisonCondition;
import com.jdbaccess.sql.LogicalCondition;
import com.jdbaccess.transfer.Row;
import com.jdbaccess.transfer.TransferObject;
import com.your_domain.your_product.address.transfer.Address;
import com.your_domain.your_product.employee.transfer.EmpAddr;
import com.your_domain.your_product.employee.transfer.Employee;

/**
 * persistence layer: dao class of entity employee 
 */
public class EmployeeDaoImpl extends DataAccessObjectImpl implements EmployeeDao {
  private static String ENTITY = Employee.getEntityName();
  private static String ENTITY_ADDRESS = Address.getEntityName();
  private static String ENTITY_EMP_ADDR = EmpAddr.getEntityName();
  private static String CREATE_TABLE_STRING = 
    "create table " + ENTITY + 
    " (id int not null auto_increment, name varchar(255) unique, salary double, description longtext, picture longblob, creation datetime, modification datetime, primary key(id)) engine = innodb";

  public EmployeeDaoImpl() {
    super(); // set the appropriate DAReader
  }

  public EmployeeDaoImpl(Transaction t) {
    super(t); // set the appropriate DAReader and transaction
  }

  public void createTable() throws ApplicationException {
    createTable(CREATE_TABLE_STRING);  
  }
  
  public void dropTable() throws ApplicationException {
    dropTable(ENTITY);  
  }
  
  public void createProceduresAndFunctions() throws ApplicationException {
    createProcedure();
    createFunction();
  }
  
  public void dropProceduresAndFunctions() throws ApplicationException {
    dropProcedure("prc_addSalaryPercent");
    dropFunction("fnc_addSalaryPercent");
  }
  
  public Employee insert(Employee emp) throws ApplicationException {
    ArrayList transferObjects = new ArrayList();
    transferObjects.add(emp);
    ArrayList retTransferObjects = (ArrayList)insert(transferObjects);
    return (Employee) retTransferObjects.get(0);
  }
  
  public ArrayList insert(ArrayList transferObjects) throws ApplicationException {
    for (int i=0; i<transferObjects.size(); i++) {
      TransferObject to = (TransferObject) transferObjects.get(i);
      to.setEntity(ENTITY);
    }
    return super.insert(transferObjects);
  }

  public Employee insert(String name, BigDecimal salary, String description, byte[] picture, Date creation, Date modification) throws ApplicationException {
    Employee emp = new Employee();
    emp.setValueModified("name", name);
    emp.setValueModified("salary", salary);
    emp.setValueModified("description", description);
    emp.setValueModified("picture", picture);
    emp.setValueModified("creation", creation);
    emp.setValueModified("modification", modification);
    return insert(emp);
  }
  
  public Long updateSalaryById(Long id, Number salary) throws ApplicationException {
    Row row = new Row();
    row.setEntity(ENTITY);
    row.setValue("id", id);
    row.setValueModified("salary", salary);
    return update(row);
  }
  
  public Long deleteAll() throws ApplicationException {
    return super.deleteAll(ENTITY);
  }
  
  public Long updateSalaryByName(String name, Number salary) throws ApplicationException {
    Row row = new Row();
    row.setEntity(ENTITY);
    row.setValue("name", name);
    row.setValueModified("salary", salary);
    ComparisonCondition nameEqualsComparison = new ComparisonCondition("name", ComparisonCondition.EQUALS, null);
    LogicalCondition nameEqualsCondition = new LogicalCondition(nameEqualsComparison, LogicalCondition.EMPTY, null);
    row.setWhereCondition(nameEqualsCondition);
    return update(row);
  }
  
  public Result findAll(ArrayList orderBy) throws ApplicationException {
    return findAll(ENTITY, Employee.class, orderBy);
  }
  
  public Employee findById(Long id) throws ApplicationException {
    return (Employee) findByFieldName(ENTITY, "id", id, Employee.class);
  }
  
  public Result findByName(String name) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "byName";
    String select = "select * from " + ENTITY + " where name = ?";
    s.setObjectType(ENTITY);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Employee.class);
    ArrayList params = new ArrayList();
    params.add(name);
    s.setParameters(params);
    Result r = s.execute();
    r.setReadLobsFull(true);
    return r;
  }
  
  public Result findBySalaryLessThan(Number salary) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "bySalary";
    String select = "select * from " + ENTITY + " where salary < ?";
    s.setObjectType(ENTITY);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Employee.class);
    ArrayList params = new ArrayList();
    params.add(salary);
    s.setParameters(params);
    Result r = s.execute();
    r.setReadLobsFull(true);
    return r;
  }
  
  public ArrayList findAddressesByEmpId(Long empId) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "addressesByEmpId";
    String select = "select * from " + ENTITY_ADDRESS + 
                   " where id in (select addrId from " + ENTITY_EMP_ADDR + " where empId = ?)";
    s.setObjectType(ENTITY_ADDRESS);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Address.class);
    ArrayList params = new ArrayList();
    params.add(empId);
    s.setParameters(params);
    Result r = s.execute();
    ArrayList addresses = r.getAllElements();
    r.close();
    return addresses;
  }
  
  private void createProcedure() throws ApplicationException {
    Update u = DAFactory.getUpdate(transaction);
    u.setObjectType("employee");
    u.setName("createProcedure");
    String createProcedure = 
      "CREATE PROCEDURE prc_addSalaryPercent (  " +
         "IN salary_percent INTEGER, " +
         "OUT successstring VARCHAR(1000), " +
         "OUT added_salary INTEGER " +
       ") " +
        "BEGIN " +
          "DECLARE done INTEGER DEFAULT 0; " +
          "DECLARE rows_processed INTEGER DEFAULT 0; " +
          "DECLARE id INTEGER; " +
          "DECLARE sal INTEGER; " +
          "DECLARE emp CURSOR FOR select id, salary from employee; " +
          "DECLARE CONTINUE HANDLER FOR SQLSTATE '02000' SET done = 1; " +
          "select * from employee; " +
          "OPEN emp; " +
          "SET added_salary = 0; " +
          "SET rows_processed = 0; " +
          "REPEAT " +
            "FETCH emp INTO id, sal; " +
            "IF NOT done THEN " +
              "SET rows_processed = rows_processed + 1; " +
              "SET added_salary = added_salary + sal / 100 * salary_percent; " +
            "END IF; " +
          "UNTIL done END REPEAT; " +
          "UPDATE employee SET salary = salary + salary / 100 * salary_percent; " +
          "IF rows_processed > 0 THEN " +
            "SELECT CONCAT('Congratulations: Update of ', rows_processed, ' employees by ', salary_percent, ' %') INTO successstring; " +
          "END IF; " +
          "select * from employee; " +
        "END;";
    u.setSql(createProcedure);
    u.execute();
    u.end();
  }

  private void createFunction() throws ApplicationException {
    Update u = DAFactory.getUpdate(transaction);
    u.setObjectType("employee");
    u.setName("createFunction");
    String createFunction = 
      "CREATE FUNCTION fnc_addSalaryPercent ( " +
          "salary_percent INTEGER " +
        ") RETURNS INTEGER " +
        "BEGIN " +
          "DECLARE done INTEGER DEFAULT 0; " +
          "DECLARE rows_processed INTEGER DEFAULT 0; " +
          "DECLARE id INTEGER; " +
          "DECLARE sal INTEGER; " +
          "DECLARE added_salary INTEGER; " +
          "DECLARE successstring VARCHAR(1000); " +
          "DECLARE emp CURSOR FOR select id, salary from employee; " +
          "DECLARE CONTINUE HANDLER FOR SQLSTATE '02000' SET done = 1; " +
          "OPEN emp; " +
          "SET added_salary = 0; " +
          "SET rows_processed = 0; " +
          "REPEAT " +
            "FETCH emp INTO id, sal; " +
            "IF NOT done THEN " +
              "SET rows_processed = rows_processed + 1; " +
              "SET added_salary = added_salary + sal / 100 * salary_percent; " +
            "END IF; " +
          "UNTIL done END REPEAT; " +
          "UPDATE employee SET salary = salary + salary / 100 * salary_percent; " +
          "IF rows_processed > 0 THEN " +
            "SELECT CONCAT('Congratulations: Update of ', rows_processed, ' employees by ', salary_percent, ' %') INTO successstring; " +
          "END IF; " +
          "CLOSE emp; " +
          "RETURN added_salary; " +
        "END;";
    u.setSql(createFunction);
    u.execute();
    u.end();
  }
}
