package com.your_domain.your_product.employee.session;

import java.util.ArrayList;

import com.jdbaccess.da.Result;
import com.jdbaccess.da.Transaction;
import com.jdbaccess.exception.ApplicationException;
import com.your_domain.your_product.address.dao.AddressDao;
import com.your_domain.your_product.address.dao.AddressDaoImpl;
import com.your_domain.your_product.address.transfer.Address;
import com.your_domain.your_product.employee.dao.EmpAddrDao;
import com.your_domain.your_product.employee.dao.EmpAddrDaoImpl;
import com.your_domain.your_product.employee.dao.EmployeeDao;
import com.your_domain.your_product.employee.dao.EmployeeDaoImpl;
import com.your_domain.your_product.employee.transfer.EmpAddr;
import com.your_domain.your_product.employee.transfer.Employee;
import com.your_domain.your_product.employee.transfer.EmployeeVO;

/**
 * Session-layer.
 * Implementation class of EmployeeSession and EmployeeSessionHome. 
 * A session object is constructed with the empty constructor when no data have to be modified else 
 * it is constructed with its underlying transaction object, so that data can be modified in 
 * that transaction.
 */
public class EmployeeSessionImpl implements EmployeeSession, EmployeeSessionHome {
  static final long serialVersionUID = EmployeeSessionImpl.class.hashCode();
  protected Transaction transaction;

  /**
   * Constructs an EmployeeSessionImpl object without a transaction
   */
  public EmployeeSessionImpl() {
  }

  /**
   * Constructs an EmployeeSessionImpl object with a transaction
   * @param transaction transaction
   */
  public EmployeeSessionImpl(Transaction transaction) {
    this.transaction = transaction;
  }

  /**
   * @see EmployeeSession#insert(EmployeeVO)
   */
  public EmployeeVO insert(EmployeeVO empVO) throws ApplicationException {
    EmployeeVO newEmpVO = new EmployeeVO();
    Employee newEmp = insert(empVO.getEmployee());
    Long newEmpDaoId = newEmp.getId();
    newEmpVO.setEmployee(newEmp);
    AddressDaoImpl addrDB = new AddressDaoImpl(transaction);
    ArrayList addressDaos = empVO.getAddresses();
    ArrayList newAddressDaos = addrDB.insert(addressDaos);
    newEmpVO.setAddresses(newAddressDaos);
    EmpAddrDaoImpl empAddrDB = new EmpAddrDaoImpl(transaction);
    for (int i=0; i<newAddressDaos.size(); i++) {
      Address newAddrDao = (Address) newAddressDaos.get(i);
      Long newAddrDaoId = newAddrDao.getId();
      EmpAddr newEmpAddrDao = new EmpAddr();
      newEmpAddrDao.setValueModified("empId", newEmpDaoId);
      newEmpAddrDao.setValueModified("addrId", newAddrDaoId);
      newEmpAddrDao.setValueModified("orderId", new Integer(i+1));
      empAddrDB.insert(newEmpAddrDao);
    }
    transaction.commit();
    return newEmpVO;
  }
  
  /**
   * @see EmployeeSession#update(EmployeeVO)
   */
  public void update(EmployeeVO empVO) throws ApplicationException {
    Employee emp = empVO.getEmployee();
    update(emp);
    ArrayList addressDaos = empVO.getAddresses();
    AddressDao addrDao = new AddressDaoImpl(transaction);
    EmpAddrDao empAddrDao = new EmpAddrDaoImpl(transaction);
    empAddrDao.deleteByEmpId(emp.getId());
    ArrayList newEmpAddresses = new ArrayList();
    for (int i=0; i<addressDaos.size(); i++) {
      Address addr = (Address) addressDaos.get(i);
      addrDao.update(addr);
      EmpAddr newEmpAddr = new EmpAddr();
      newEmpAddr.setValueModified("empId", emp.getId());
      newEmpAddr.setValueModified("addrId", addr.getId());
      newEmpAddr.setValueModified("orderId", new Integer(i+1));
      newEmpAddresses.add(newEmpAddr);
    }
    empAddrDao.insert(newEmpAddresses);
    transaction.commit();
  }
  
  /**
   * @see EmployeeSession#findById(Long)
   */
  public EmployeeVO findById(Long empId) throws ApplicationException {
    EmployeeVO empVO = null;
    Employee emp = findEmployeeById(empId);
    if (emp != null) {
      empVO = new EmployeeVO();
      empVO.setEmployee(emp);
    }
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    ArrayList addresses = empDao.findAddressesByEmpId(empId);
    empVO.setAddresses(addresses);
    return empVO;
  }

  /**
   * @see EmployeeSession#findByName(String)
   */
  public EmployeeVO findByName(String empName) throws ApplicationException {
    EmployeeVO empVO = null;
    Result r = findEmployeeByName(empName);
    Employee emp = (Employee) r.getFirstElement();
    r.close();
    if (emp != null) {
      empVO = new EmployeeVO();
      empVO.setEmployee(emp);
      EmployeeDao empDao = new EmployeeDaoImpl(transaction);
      ArrayList addresses = empDao.findAddressesByEmpId(emp.getId());
      empVO.setAddresses(addresses);
    }
    return empVO;
  }
  
  /**
   * @see EmployeeSession#findEmployeeAll(ArrayList)
   */
  public Result findEmployeeAll(ArrayList orderBy) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    return empDao.findAll(orderBy);
  }
  
  /**
   * @see EmployeeSession#findEmployeeById(Long)
   */
  public Employee findEmployeeById(Long id) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    return empDao.findById(id);
  }
  
  /**
   * @see EmployeeSession#findEmployeeByName(String)
   */
  public Result findEmployeeByName(String name) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    return empDao.findByName(name);
  }
  
  /**
   * @see EmployeeSession#findEmployeeBySalaryLessThan(Number)
   */
  public Result findEmployeeBySalaryLessThan(Number salary) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    return empDao.findBySalaryLessThan(salary);
  }
  
  private Employee insert(Employee emp) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    Employee newEmp = (Employee) empDao.insert(emp);
    return newEmp;
  }
  
  private Long update(Employee emp) throws ApplicationException {
    EmployeeDao empDao = new EmployeeDaoImpl(transaction);
    Long count = empDao.update(emp);
    return count;
  }
  
}
